/* ============================================================
 * File  : guielements.cpp
 * Author: Eric Giesselbach <ericgies@kabelfoon.nl>
 * Date  : 2004-02-12
 * Description : streamtuned gui
 *
 *
 * Copyright 2003 by Eric Giesselbach

 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published bythe Free Software Foundation;
 * either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * ============================================================ */


#include <qapplication.h>
#include <unistd.h>
#include <qpushbutton.h>
#include <qfont.h>
#include <qpainter.h>
#include <qlabel.h>
#include <qimage.h>

#include "guielements.h"

using namespace std;

static QString emptyStr = "";


// ----------------------------------------------------------------------

Gauge::Gauge(QWidget *parent, GuiParameters* params, unsigned int x, unsigned int y, unsigned int w, unsigned int h)
     : QWidget( parent )
{
   guiParameters = params;
   percentage = 0;
   setGeometry( x, y, w, h );
   setBackgroundOrigin( QWidget::WindowOrigin );

   int barCount;

   // we want 20 bars, space width = 1
   barCount   = 20;
   barWidth = int ( ( w - 19 ) / 20 );

   // bars must have width > 1
   if ( barWidth < 2 )
   {
     // number of bars width = 2 (< 20)
     barCount   = int ( w / 3 );
     barWidth = 2;
   }

   if ( barCount == 0 )
   {
     // no gauge
     barCount = 1;
     barWidth = 0;
   }

   gaugeWidth = barCount * barWidth + barCount - 1;
   div = 100 / barCount;
}

void Gauge::paintEvent( QPaintEvent * )
{

    QPainter painter( this );

    painter.setPen( guiParameters->gaugeColor );
    painter.setBrush( QBrush(guiParameters->gaugeColor, SolidPattern) );

    int barCount = int ( percentage / div );

    for ( int i = 0; i < barCount; i++)
    {
      painter.drawRect( i*(barWidth + 1), 0, barWidth, height());
    }

    painter.setPen( black );
    painter.setBrush( QBrush(black, NoBrush) );
    painter.drawRect( 0, 0, gaugeWidth, height() );
}


void Gauge::setPercentage(int percent)
{
   percentage = percent;
   update();
}

// ----------------------------------------------------------------------

FFTBox::FFTBox(QWidget *parent, GuiParameters* params, int barcnt, unsigned int x, unsigned int y, unsigned int w, unsigned int h)
     : QWidget( parent, "", Qt::WRepaintNoErase )
{
   guiParameters = params;
   percentage = 0;
   go = false;

   // we want 20 bars, space width = 1
   barCount   = barcnt;
   if (barCount > 100)
   {
     cerr << "gauge supports 100 bars max" << endl;
     exit(-1);
   }

   barWidth = int ( ( w - barCount + 1 ) /barCount );

   // bars must have width > 0
   if ( barWidth < 1 )
     barWidth = 1;

   gaugeWidth  = barCount * barWidth + barCount + 1;// - 1;
   div =  float(h) / 100;

   setGeometry( x, y, gaugeWidth, h );
   setBackgroundOrigin( QWidget::WindowOrigin );

   QFontMetrics metric( font() );
   textPosX = ( gaugeWidth - metric.width("no data") ) / 2;
   textPosY = height() - 5;

   fftPict.load( QString(PREFIX"/share/streamtuned/s_fft.png") );

   if ( fftPict.isNull() )
   {
     cerr << "warning: could not load s_fft.png (fft background)" << endl;
     fftPict = QPixmap(size());
     fftPict.fill(guiParameters->gaugeColor);
   }

   fftMask = QBitmap(fftPict.size(), true);
   bckPict = QPixmap::grabWidget(parent, x, y, gaugeWidth, h);
   drawPict = QPixmap(size());
   go = true;
}

FFTBox::~FFTBox()
{
}


void FFTBox::paintEvent( QPaintEvent * )
{
    if (!go) return;
    QColor col = Qt::color1;

    QPainter *painter;

    fftMask.fill(Qt::color0);
    painter = new QPainter( &fftMask );

    int barHeight;

    painter->setPen( col );
    painter->setBrush( QBrush(col, SolidPattern) );

    if (percentage)
    {
        for ( int i = 0; i < barCount; i++)
        {
          barHeight = int ( float( percentage->values[i] ) * div );
          if ( barHeight > height() ) barHeight = height();
          if ( barHeight < 0 ) barHeight = 0;
          painter->drawRect( i*(barWidth + 1) + 1, height() - barHeight, barWidth, height() );
        }
    }

    painter->setPen( Qt::color1 );
    painter->setBrush( QBrush(Qt::color1, NoBrush) );
    painter->drawRect( 0, height()-1, gaugeWidth, height() );
    delete(painter);

    fftPict.setMask(fftMask);

    bitBlt(&drawPict, QPoint(0,0), &bckPict, rect(), Qt::CopyROP);
    bitBlt(&drawPict, QPoint(0,0), &fftPict, rect(), Qt::CopyROP);
    bitBlt(this, QPoint(0,0), &drawPict, rect(), Qt::CopyROP);
}

void FFTBox::setPercentage(Spectrum* percent)
{
   percentage = percent;
   update();
}

// ----------------------------------------------------------------------

Led::Led(QWidget *parent, unsigned int x, unsigned int y, bool tilt)
     : QWidget( parent )
{
   blinkHi = 200;
   blinkLo = 50;
   blinkIsHi = true;

   blinkTimer = new QTimer( this );
   connect( blinkTimer, SIGNAL(timeout()),
            this, SLOT(toggleBlink()) );

   tilted = tilt;
   setColor( QColor(red) );

   setGeometry( x, y, 42, 17 );
   setBackgroundOrigin( QWidget::WindowOrigin );

   if ( tilted )
     points.setPoints( 3, 0,0, 42,0, 42,17 );
   else
     points.setPoints( 3, 0,0, 42,17, 0,17 );
}

void Led::paintEvent( QPaintEvent * )
{
    QPainter painter( this );

    QBrush brush( color, Qt::SolidPattern );
    painter.setBrush( brush );
    painter.setPen( color );
    painter.drawPolygon( points );
}

void Led::setLightness(int light)
{
   setBlinking(false);
   v = light;
   color.setHsv(h, s, v);
   update();
}

void Led::setBlinking(bool blink)
{
   if ( blink )
     blinkTimer->start( 300, false);
   else
     blinkTimer->stop();
}

void Led::setColor(QColor col)
{
   color = col;
   color.hsv(&h, &s, &v);
   update();
}

void Led::toggleBlink()
{
  if ( blinkIsHi )
    color.setHsv(h, s, blinkLo);
  else
    color.setHsv(h, s, blinkHi);

  update();
  blinkIsHi = !blinkIsHi;
}

// ----------------------------------------------------------------------

PlayerMessageBox::PlayerMessageBox(QWidget *parent, GuiParameters* params, unsigned int x, unsigned int y,
                                   unsigned int w)
     : QWidget( parent )
{
   guiParameters = params;
   hideTimer = new QTimer(this);

   connect( hideTimer, SIGNAL(timeout()),
            this, SLOT(clearMessages()) );

   setGeometry( x, y, w, 54 );
   setBackgroundOrigin( QWidget::WindowOrigin );

   box.setPoints( 4, 42,0, w-1,0, w-1,52, 42,52);
   lab.setPoints( 6, 3,8, 27,8, 27,14, 41,14, 27,14, 27,20);

   line1 = prepLabel( new QLabel(this), 0, w);
   line2 = prepLabel( new QLabel(this), 1, w);
   line3 = prepLabel( new QLabel(this), 2, w);
}

void PlayerMessageBox::paintEvent( QPaintEvent * )
{
    QPainter painter( this );

    //QBrush brush( white, Qt::SolidPattern );
    //painter.setBrush( brush );
    painter.setPen( guiParameters->playerBoxEdge );
    painter.drawPolygon( box );
    painter.drawPolygon( lab );
}

QLabel* PlayerMessageBox::prepLabel(QLabel *label, unsigned int index, unsigned int w)
{
    label->setText( "" );
    label->setPaletteForegroundColor( guiParameters->playerBoxFont );
    label->setGeometry( 52, index*17 + 1, w-58, 17 );
    label->setBackgroundOrigin( QWidget::WindowOrigin );
    return label;
}

void PlayerMessageBox::addMessage(QString message)
{
    QStringList list = QStringList::split("\n", message);
    QStringList::Iterator i;
   
    for ( i = list.begin(); i != list.end(); ++i )
    {
        if ( *i
              && *i != ""
              && *i != line1->text()
              && *i != line2->text()
              && *i != line3->text() )
        {
          hideTimer->changeInterval(8000);
          if ( !isShown() ) show();
          line3->setText( line2->text() );
          line2->setText( line1->text() );
          line1->setText( *i );
        }
      }
}

void PlayerMessageBox::clearMessages()
{
    hide();
    line1->setText("");
    line2->setText("");
    line3->setText("");
}

// ----------------------------------------------------------------------

HarvesterMessageBox::HarvesterMessageBox(QWidget *parent, GuiParameters* params, unsigned int x, unsigned int y,
                                   unsigned int w)
     : QWidget( parent )
{
   guiParameters = params;

   setGeometry( x, y, w, 20 );
   setBackgroundOrigin( QWidget::WindowOrigin );

   box.setPoints( 4, 3,0, w-40,0, w-40,19, 3,19);
   lab.setPoints( 6, w-1,14, w-25,14, w-25,9, w-40,9, w-25,9, w-25,2);

   line = prepLabel( new QLabel(this), w);
}

void HarvesterMessageBox::paintEvent( QPaintEvent * )
{
    QPainter painter( this );

    //QBrush brush( white, Qt::SolidPattern );
    //painter.setBrush( brush );
    painter.setPen( guiParameters->harvesterBoxEdge );
    painter.drawPolygon( box );
    painter.drawPolygon( lab );
}

QLabel* HarvesterMessageBox::prepLabel(QLabel *label, unsigned int w)
{
    label->setText( "" );
    label->setPaletteForegroundColor( guiParameters->playerBoxFont );
    label->setGeometry( 9, 1, w-58, 18 );
    label->setBackgroundOrigin( QWidget::WindowOrigin );
    return label;
}

void HarvesterMessageBox::setMessage(QString message)
{
   if ( (!message || message == "") && isShown() ) hide();
   line->setText( message );
   if ( message && message != "" && !isShown() ) show();
}


// ----------------------------------------------------------------------

RecLed::RecLed(QWidget *parent, unsigned int x, unsigned int y) : QWidget( parent )
{
   indicator = false;

   setColor( QColor(red) );

   setGeometry( x, y, 12, 12 );
   setBackgroundOrigin( QWidget::WindowOrigin );

   circle = QRect(0, 0, 11, 11);
}

void RecLed::paintEvent( QPaintEvent * )
{
    if (indicator)
    {
      QPainter painter( this );
      QBrush brush( color, Qt::SolidPattern );
      painter.setBrush( brush );
      painter.setPen( darkGray );
      painter.drawEllipse( circle );
    }
}

void RecLed::setColor(QColor col)
{
   color = col;
   color.hsv(&h, &s, &v);
   update();
}

void RecLed::setLightness(int light)
{
   v = light;
   color.setHsv(h, s, v);
   update();
}


void RecLed::setIndicator(bool on)
{
   indicator = on;
   update();
}

// ----------------------------------------------------------------------

Logo::Logo(QWidget *parent, int x, int y, int w, int h)
            : QLabel( parent )
{
    setText( "StreamTuned" );
    setPaletteForegroundColor( gray );
    setGeometry( x, y, w, h );
    setBackgroundOrigin( QWidget::WindowOrigin );
    //setFrameStyle( QFrame::Box | QFrame::Plain );
    //setLineWidth( 1 );

    QFont normFont = font();
    //boldFont = font();
    normFont.setBold( true );
    setFont( normFont );
}

void Logo::parentHasFocus( bool hasFocus)
{
   if ( hasFocus )
     setPaletteForegroundColor( darkGray );
   else
     setPaletteForegroundColor( lightGray );
}

// ----------------------------------------------------------------------

MessageLabel::MessageLabel(QWidget *parent, GuiParameters* params, int x, int y, int w, int h)
            : QLabel( parent )
{
    guiParameters = params;
    hide();
    setText( "" );
    setPaletteForegroundColor( green );
    setGeometry( x, y, w, h );
    setBackgroundOrigin( QWidget::WindowOrigin );
    setFrameStyle( QFrame::Box | QFrame::Plain );
    setLineWidth( 1 );

    normFont = font();
    boldFont = font();
    boldFont.setBold( true );
    setFont( boldFont );
}


// ----------------------------------------------------------------------

ItemLabel::ItemLabel(QWidget *parent, GuiParameters* params, int x, int y, int w, int h)
            : QLabel( parent )
{
   guiParameters = params;
    setText( "" );
    setPaletteForegroundColor( guiParameters->textLoColor );
    setGeometry( x, y, w, h );
    setBackgroundOrigin( QWidget::WindowOrigin );
    normFont = font();
    boldFont = font();
    boldFont.setBold( true );
    setFont( normFont );
}

void ItemLabel::setSelected(bool cursor)
{
    if ( cursor )
      setPaletteForegroundColor( guiParameters->textSelectedColor );
    else
      setPaletteForegroundColor( guiParameters->textLoColor );

    selected = cursor;
}

void ItemLabel::setHighLight(bool cursor)
{
    if ( cursor )
    {
      setPaletteForegroundColor( guiParameters->textHighlightColor );
      setFont( boldFont );
    }
    else
    {
      setPaletteForegroundColor( guiParameters->textLoColor );
      setFont( normFont );
    }

    selected = cursor;
}

void ItemLabel::setColor(QColor& color, bool bold)
{
      setPaletteForegroundColor( color );
      
      if (bold)
        setFont( boldFont );
      else
        setFont( normFont );
}

// ----------------------------------------------------------------------

// ----------------------------------------------------------------------

ItemIcon::ItemIcon(QWidget *parent, GuiParameters* params, int x, int y, int w, int h)
            : QLabel( parent )
{
    guiParameters = params;
    setText( "" );
    setPaletteForegroundColor( guiParameters->textLoColor );
    setGeometry( x, y, w, h );
    setBackgroundOrigin( QWidget::WindowOrigin );
}

void ItemIcon::setSelected(bool cursor)
{
    if ( cursor )
      setPaletteForegroundColor( guiParameters->textSelectedColor );
    else
      setPaletteForegroundColor( guiParameters->textLoColor );

    selected = cursor;
}

void ItemIcon::setHighLight(bool cursor)
{
    if ( cursor )
    {
      setPaletteForegroundColor( guiParameters->textHighlightColor );
    }
    else
    {
      setPaletteForegroundColor( guiParameters->textLoColor );
    }

    selected = cursor;
}

// ----------------------------------------------------------------------

FolderLabel::FolderLabel(QWidget *parent, GuiParameters* params, int x, int y, int w, int h)
            : QLabel( parent )
{
    guiParameters = params;
    setText( "" );
    setPaletteForegroundColor( white );
    setGeometry( x, y, w, h );
    setBackgroundOrigin( QWidget::WindowOrigin );
    setAlignment(AlignHCenter | AlignVCenter);
    normFont = font();
    boldFont = font();
    boldFont.setBold( true );
    setFont( normFont );
}

void FolderLabel::setSelected(bool cursor)
{
    if ( cursor )
    {
      setPaletteForegroundColor( black );
      setPaletteBackgroundColor( gray );
    }
      else
    {
      unsetPalette();
      setPaletteForegroundColor( white );
      setBackgroundOrigin( QWidget::WindowOrigin );
    }

    selected = cursor;
}

void FolderLabel::setHighLight(bool cursor)
{
    if ( cursor )
    {
      setPaletteForegroundColor( white );
      setFont( boldFont );
    }
    else
    {
      setPaletteForegroundColor( guiParameters->textLoColor );
      setFont( normFont );
    }

    selected = cursor;
}

